#include <stdio.h>
#include <stdlib.h>
#include <memory.h>
#include "aes.h"

const int   NO_ERROR = 0;
const int   PADDING_ERROR = 1;
const int   MEMORY_ERROR = 2;
const int   OTHER_DECRYPTION_ERROR = 3;

/**
 * @brief performServerDecrypt function simulates remote server decryption.
 * Server decrypts input data using AES in CBC mode with embeeded key
 * and verify proper PKCS#7 padding
 * @param input input encrypted data
 * @param length length of input data
 * @param iv initialization vector (16B)
 * @return status NO_ERROR if decryption succeed, error otherwise
 */
int performServerDecrypt(const unsigned char* input, const int length, const unsigned char iv[16]) {
    int status = NO_ERROR;

    aes_context ctx;
    unsigned char secret_server_key[] = "Very secret key!"; // 5665727920736563726574206b657921
    unsigned char initVector[16];

    unsigned char* buffer = (unsigned char*) malloc(length);
    if (buffer == NULL) { status = MEMORY_ERROR; }

    // Create local copy of input data so we will not leak decrypted value
    if (status == NO_ERROR) {
        memcpy(buffer, input, length);
        memcpy(initVector, iv, sizeof(initVector));
    }

    // Initialize key (never shown outside "server")
    if (status == NO_ERROR) {
        if (aes_setkey_dec(&ctx, secret_server_key, 128) != 0) {
            status = OTHER_DECRYPTION_ERROR;
        }
    }
    // Perform decryption
    if (status == NO_ERROR) {
        aes_crypt_cbc(&ctx, AES_DECRYPT, length, initVector, buffer, buffer);
    }

    // Check PKCS#7 padding
    unsigned char paddingLen = 0;
    if (status == NO_ERROR) {
        paddingLen = buffer[length - 1];
        if ((paddingLen > 16) || (paddingLen <= 0)) { status = PADDING_ERROR; }
    }

    if (status == NO_ERROR) {
        // Check that all padding values are properly set
        for (int offset = length - 1; offset >= length - paddingLen; offset--) {
            if (buffer[offset] != paddingLen) {
                status = PADDING_ERROR;
                break;
            }
        }
    }


    if (status == NO_ERROR) {
        // Perform server operation on decrypted data... nothing here
    }

    // Clear memory with sensitive data
    memset(secret_server_key, sizeof(secret_server_key), 0);
    memset(&ctx, sizeof(aes_context), 0);
    if (buffer != NULL) {
        memset(buffer, length, 0); // clear memory before release
        free(buffer);
    }

    // Return status
    return status;
}

// Ciphertext captured by an attacker: B2EC2BCA09E2A8597FAD37A88C8BCD90F6AB50AFC31B6CCB28941F50AFA2CEFC
unsigned char intercepted_ciphertext[] = {0xB2, 0xEC, 0x2B, 0xCA, 0x09, 0xE2, 0xA8, 0x59, 0x7F, 0xAD, 0x37, 0xA8, 0x8C, 0x8B, 0xCD, 0x90, 0xF6, 0xAB, 0x50, 0xAF, 0xC3, 0x1B, 0x6C, 0xCB, 0x28, 0x94, 0x1F, 0x50, 0xAF, 0xA2, 0xCE, 0xFC};
const int ciphertextLen = sizeof(intercepted_ciphertext);

int main() {
    unsigned char    initVect[16];
    memcpy(initVect, "Unique init vect", 16); // 556e6971756520696e69742076656374

    printf("Going to send input for decryption to 'remote server'...\n");
    int status = performServerDecrypt(intercepted_ciphertext, ciphertextLen, initVect);
    if (status == NO_ERROR) {
        printf("Decryption suceeded \n");
    }
    else {
        printf("Decryption failed with %d status \n", status);
    }

    getchar(); // to prevent immediate closing of cmd window
    return 0;
}
